from dune.perftool.error import PerftoolLoopyError
from dune.perftool.generation import (generator_factory,
                                      get_counted_variable,
                                      get_global_context_value,
                                      temporary_variable,
                                      )


class FlipFlopBuffer(object):
    def __init__(self, identifier, base_storage_size=None, num=2):
        self.identifier = identifier
        self.base_storage_size = base_storage_size
        self.num = num

        # Initialize the counter that switches between the base storages!
        self._current = 0

        # Generate the base storage names
        self.base_storage = tuple("{}_base_{}".format(self.identifier, i) for i in range(self.num))

    def switch_base_storage(self):
        self._current = (self._current + 1) % self.num

    def get_temporary(self, **kwargs):
        assert("base_storage" not in kwargs)
        assert("storage_shape" not in kwargs)

        # Select the base storage and increase counter
        base = self.base_storage[self._current]

        # Construct a temporary name
        name = kwargs.pop("name", get_counted_variable(self.identifier))
        if name is None:
            from pudb import set_trace; set_trace()

        # Get geometric dimension
        formdata = get_global_context_value('formdata')
        dim = formdata.geometric_dimension

        # Construct the temporary and return it
        temporary_variable(name,
                           base_storage=base,
                           managed=True,
                           **kwargs
                           )

        return name


@generator_factory(item_tags=("buffer"), cache_key_generator=lambda i, **kw: i, context_tags=("kernel",))
def initialize_buffer(identifier, base_storage_size=None, num=2):
    if base_storage_size is None:
        raise PerftoolLoopyError("The buffer for identifier {} has not been initialized.".format(identifier))
    return FlipFlopBuffer(identifier,
                          base_storage_size=base_storage_size,
                          num=num,
                          )


def get_buffer_temporary(identifier, **kwargs):
    return initialize_buffer(identifier).get_temporary(**kwargs)


def switch_base_storage(identifier):
    initialize_buffer(identifier).switch_base_storage()
